<?php

namespace ppp;

use Exception;

/**
 * 通知用のActivityを扱うクラス。
 * 記事へのlike付与も扱う。
 */
class Activitys
{
    private $db;

    public function __construct($db)
    {
        $this->db = new SQLiteCRUD($db);
        $this->initialize();
    }

    private function initialize()
    {
        $query = "
            CREATE TABLE IF NOT EXISTS activity (
                [id] INTEGER PRIMARY KEY AUTOINCREMENT,
                [target] INTEGER,
                [created_at] DATETIME,
                [from_ip] TEXT,
                [action] TEXT,
                [read] BOOLEAN DEFAULT '0'
            )
        ";
        $this->db->execute($query);
    }

    /**
     * アクティビティを記録する
     *
     * @param string $action    アクション内容。like, comment
     * @param int $article_id   対象記事ID。対象がない場合は-1。
     * @param bool $use_transaction トランザクションを使用するか。通常true。呼び出し元でトランザクションを使用している場合はfalse。
     *
     * @return void
     *
     */
    public function addActivity(string $action, int $article_id, bool $use_transaction = true): void
    {
        try {
            if ($use_transaction) {
                $this->db->begin();
            }
            date_default_timezone_set('Asia/Tokyo');
            $date = date("Y-m-d H:i:s");
            $data = [
                'target' => $article_id,
                'action' => $action,
                'from_ip' => $_SERVER['REMOTE_ADDR'] ?? "unknonw",
                'created_at' => $date
            ];
            $this->db->insertOrUpdate('activity', $data);

            //古いアクティビティを削除
            $before7d = date('Y-m-d H:i:s', strtotime('-7 days', strtotime($date)));  // 7日前の日時を計算して取得
            $this->db->execute("delete from activity where created_at < :beforeTime", ["beforeTime" => $before7d]);
            if ($use_transaction) {
                $this->db->commit();
            }
        } catch (Exception $e) {
            if ($use_transaction) {
                $this->db->rollback();
            }
            throw $e;
        }
    }

    //同一IDから同一アクションが24時間以内にあったか確認する
    public function duplicateCheck(string $action, int $article_id): bool
    {
        date_default_timezone_set('Asia/Tokyo');
        $date = date("Y-m-d H:i:s");
        $before24h = date('Y-m-d H:i:s', strtotime('-24 hours', strtotime($date)));  // 24時間前の日時を計算して取得
        $rows = $this->db->search(
            "activity",
            "",
            "target=:id and from_ip = :from and action = :action and created_at > :beforeTime",
            [
                "id" => $article_id,
                "from" => $_SERVER['REMOTE_ADDR'],
                "action" => $action,
                "beforeTime" => $before24h
            ],
            0,
            1
        );

        if (count($rows) === 0) {
            return false;
        }
        return true;
    }


    /**
     * いいね増加処理
     *
     * @param int $article_id   記事ID
     *
     * @return int  増加後のいいね数
     *
     */
    public function addLike(int $article_id): int
    {
        //24時間以内に同一操作がある場合は加算せず今の数値を返す
        if ($this->duplicateCheck("like", $article_id)) {
            //現在のイイネ数を取得して返す
            $like = $this->getLike($article_id);
            return $like;
        }
        try {
            $this->db->begin();
            //記事のイイネ数を増加してアクティビティ記録
            $this->db->execute("update articles set like = (like + 1) where id=:id", ["id" => $article_id]);
            $this->addActivity("like", $article_id, false);
            $this->db->commit();
            $like = $this->getLike($article_id); //増加のイイネ数を取得
        } catch (Exception $e) {
            $this->db->rollback();
            throw $e;
        }
        return $like;
    }

    /**
     * イイネ数取得
     *
     * @param int $article_id   記事ID
     *
     * @return int  現在のイイネ数
     *
     */
    public function getLike(int $article_id): int
    {
        $rows = $this->db->search("articles", "", "id=:id", ["id" => $article_id]);
        if (count($rows) > 0) {
            return $rows[0]["like"] ?? 0;
        }
        return 0;
    }
}
