<?php

namespace ppp;

/**
 * コメントを扱うクラス。
 */
class Comments
{
    private $sqlcrud;
    private $db;

    public function __construct($db)
    {
        $this->db = $db;
        $this->sqlcrud = new SQLiteCRUD($db);
        $this->initialize();
    }

    private function initialize()
    {
        $query = "
            CREATE TABLE IF NOT EXISTS comments (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                article_id INTEGER NOT NULL,
                parent_id INTEGER DEFAULT NULL,
                name TEXT NOT NULL,
                email TEXT,
                comment TEXT NOT NULL,
                from_ip TEXT,
                created_at DATETIME,
                is_approved INTEGER DEFAULT 0,
                from_admin INTEGER DEFAULT 0,
                read INTEGER DEFAULT 0
            )
        ";
        $this->sqlcrud->execute($query);
    }

    public function addComment($article_id, $name, $email, $comment, $parent_id = null, $from_admin = false)
    {
        $approved = false;
        if ($from_admin) {
            $approved = true;
        }
        date_default_timezone_set("Asia/Tokyo");
        $date = date("Y-m-d H:i:s");
        $data = [
            "article_id" => $article_id,
            "parent_id" => $parent_id,
            "name" => $name,
            "email" => $email,
            "comment" => $comment,
            "from_admin" => $from_admin,
            "created_at" => $date,
            "is_approved" => $approved,
            'from_ip' => $_SERVER['REMOTE_ADDR'] ?? "unknonw"
        ];
        $this->sqlcrud->insertOrUpdate("comments", $data);

        //アクティビティを記録
        if (!$from_admin) {
            $activity = new Activitys($this->db);
            $activity->addActivity("comment", $article_id);
        }
    }

    public function deleteComment($comment_id)
    {
        $this->sqlcrud->delete("comments", $comment_id);
        $this->sqlcrud->rawQuery("DELETE FROM comments WHERE parent_id = :parent_id", ["parent_id" => $comment_id]);
    }

    public function approveComment($comment_id)
    {
        $this->sqlcrud->insertOrUpdate("comments", ["id" => $comment_id, "is_approved" => 1]);
    }

    public function disapproveComment($comment_id)
    {
        $this->sqlcrud->insertOrUpdate("comments", ["id" => $comment_id, "is_approved" => 0]);
    }

    public function readComment($comment_id)
    {
        $this->sqlcrud->insertOrUpdate("comments", ["id" => $comment_id, "read" => 1]);
    }

    public function disreadComment($comment_id)
    {
        $this->sqlcrud->insertOrUpdate("comments", ["id" => $comment_id, "read" => 0]);
    }

    public function getComments($limit, $page, $sort, $query = "", $values = [])
    {
        $offset = $limit * $page;
        return $this->sqlcrud->search("comments", $sort, $query, $values, $offset, $limit);
    }
    public function getCommentsWithTotalCount($limit, $page, $sort, $query = "", $values = [])
    {
        return [
            $this->getComments($limit, $page, $sort, $query, $values),
            $this->sqlcrud->getCount("comments",  $query, $values)
        ];
    }
    public function getUnreadComments($limit, $page, $sort, $query = "", $values = [])
    {
        $and = "";
        if ($query != "") {
            $and = " and ";
        }
        $query = $query . $and . "read = false";
        $offset = $limit * $page;
        return $this->sqlcrud->search("comments", $sort, $query, $values, $offset, $limit);
    }
    public function getUnreadCommentCount($query = "", $values = [])
    {
        $and = "";
        if ($query != "") {
            $and = " and ";
        }
        $query = $query . $and . "read = false";
        return $this->sqlcrud->getCount("comments",  $query, $values);
    }
    public function getUnreadCommentsWithTotalCount($limit, $page, $sort, $query = "", $values = [])
    {
        $and = "";
        if ($query != "") {
            $and = " and ";
        }
        $query = $query . $and . "read = false";
        return [
            $this->getUnreadComments($limit, $page, $query, $values),
            $this->sqlcrud->getCount("comments",  $query . " " . $sort, $values)
        ];
    }


    /**
     * 指定された記事IDに基づくコメント一覧を取得する
     *
     * @param int $articleId    記事ID。システムコメントの場合 -1。
     * @param bool $show_unapproved 承認されていないコメントを表示するか。デフォルトfalse。
     * @param int   $count  取得件数。デフォルト全件。
     * @param int   $page   取得するページ。
     * @return array    コメントツリーを格納した配列
     *
     */

    public function getCommentsByArticleId(int $article_id, bool $show_unapproved = false): array
    {
        return $this->fetchComments($article_id, $show_unapproved);
    }
    private function fetchComments(int $article_id, bool $show_unapproved = false, int $parent_id = -1)
    {
        $query = "";
        if ($show_unapproved) {
            $query =  "article_id = :article_id AND  parent_id = :parent_id";
        } else {
            $query =  "article_id = :article_id AND is_approved = 1 AND  parent_id = :parent_id";
        }
        $result = [];
        $comments = $this->sqlcrud->search("comments", "", $query . " ORDER BY created_at ASC", ["article_id" => $article_id, "parent_id" => $parent_id]);
        foreach ($comments as $comment) {
            $comment["replies"] = $this->fetchComments($article_id, $show_unapproved, $comment["id"]);
            $result[] = $comment;
        }
        return $result;
    }
}
