<?php

namespace ppp;

use SQLite3;

class SQLiteCRUD
{
    private $db;

    public function __construct($db)
    {
        $this->db = $db;
        $this->db->busyTimeout(5000);
    }

    public function begin()
    {
        $this->db->exec('begin Immediate');
    }
    public function commit()
    {
        $this->db->exec('commit');
    }
    public function rollback()
    {
        $this->db->exec('rollback');
    }
    /**
     * 指定されたテーブルへインサートまたは更新する
     *
     * $dataにキー"ID"が存在する場合、そのID行を更新する。
     * ない場合はインサートとなる
     *
     * @param string $table テーブル名
     * @param array $data   フィールド名：値のキーバリューペア
     *
     * @return int  新規行のID
     *
     */
    public function insertOrUpdate(string $table, array $data): int
    {
        $keys = array_keys($data);
        $idKey = array_search('id', $keys);
        if ($idKey !== false && isset($data['id'])) {
            $updateFields = array_map(fn ($key) => "$key = :$key", $keys);
            $updateQuery = "UPDATE $table SET " . implode(', ', $updateFields) . " WHERE id = :id";
            $stmt = $this->db->prepare($updateQuery);
        } else {
            $insertFields = implode(', ', $keys);
            $insertValues = ':' . implode(', :', $keys);
            $insertQuery = "INSERT INTO $table ($insertFields) VALUES ($insertValues)";
            $stmt = $this->db->prepare($insertQuery);
        }
        foreach ($data as $key => $value) {
            $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        }
        $stmt->execute();

        if ($idKey !== false && isset($data['id'])) {
            $result_id = $data['id'];
        } else {
            $result_id = $this->db->lastInsertRowID();
        }
        return $result_id;
    }

    /**
     * 指定されたテーブルへインサートまたは更新する
     *
     *
     * @param string $table テーブル名
     * @param string $update_key 上書き条件となるフィールド名
     * @param array $data   フィールド名：値のキーバリューペア
     *
     * @return void
     *
     */
    public function insertOrUpdateByKey(string $table, string $update_key, array $data): void
    {
        $keys = array_keys($data);
        $updateFields = array_map(fn ($key) => "$key = :$key", $keys);
        $updateQuery = "UPDATE $table SET " . implode(', ', $updateFields) . " WHERE $update_key = :update_key";
        $stmt = $this->db->prepare($updateQuery);

        foreach ($data as $key => $value) {
            $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        }
        $stmt->bindValue(":update_key", $data[$update_key], is_int($data[$update_key]) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        $stmt->execute();

        if ($this->db->changes() == 0) {
            $insertFields = implode(', ', $keys);
            $insertValues = ':' . implode(', :', $keys);
            $insertQuery = "INSERT INTO $table ($insertFields) VALUES ($insertValues)";
            $stmt = $this->db->prepare($insertQuery);

            foreach ($data as $key => $value) {
                $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
            }
            $stmt->execute();
        }
    }

    /**
     * 指定されたテーブルから指定したIDの行を削除する
     *
     * @param string $table データを削除するテーブル
     * @param int    $id    削除する行のID
     *
     * @return void
     *
     */
    public function delete(string $table, int $id): void
    {
        $deleteQuery = "DELETE FROM $table WHERE id = :id";
        $stmt = $this->db->prepare($deleteQuery);
        $stmt->bindValue(':id', $id, SQLITE3_INTEGER);
        $stmt->execute();
    }

    /**
     * 指定された条件の検索を行う
     *
     * @param string $table     テーブル名
     * @param string $sort      ソート条件
     * @param string $query     Where以降の検索条件。引数を :arg の形で含めることができる
     * @param array $conditions $queryに指定された引数の内容。[arg=>value]の配列。
     * @param int|null $offset  offset
     * @param int|null $limit   limit
     *
     * @return array
     *
     * 例）
     * search("articles", "date > :date_start and date < :date_end and tags like '* :tag *'", [date_start='2024/05/01', date_end='2024/05/01', 'log'] , 0, 0 )
     *
     */
    public function search(string $table, string $sort = "", string $query = "", array $conditions = [], int $offset = null, int $limit = null): array
    {
        $sortClause = $sort ? " order by $sort" : "";
        $whereClause = $query ? " WHERE $query" : "";
        $limitClause = $limit !== null ? " LIMIT :limit" : "";
        $offsetClause = $offset !== null ? " OFFSET :offset" : "";

        $searchQuery = "SELECT * FROM $table$whereClause$sortClause$limitClause$offsetClause";
        $stmt = $this->db->prepare($searchQuery);

        foreach ($conditions as $key => $value) {
            $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        }

        if ($limit !== null) {
            $stmt->bindValue(':limit', $limit, SQLITE3_INTEGER);
        }

        if ($offset !== null) {
            $stmt->bindValue(':offset', $offset, SQLITE3_INTEGER);
        }

        $result = $stmt->execute();
        $data = [];
        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
            $data[] = $row;
        }
        return $data;
    }

    /**
     * 指定された条件に合致するレコード数を取得する
     *
     * @param string $table     テーブル名
     * @param string $query     Where以降の検索条件。引数を :arg の形で含めることができる
     * @param array $conditions $queryに指定された引数の内容。[arg=>value]の配列。
     * @param int|null $offset  offset
     * @param int|null $limit   limit
     *
     * @return int
     *
     * 例）
     * getCount("articles", "date > :date_start and date < :date_end and tags like '* :tag *'", [date_start='2024/05/01', date_end='2024/05/01', 'log'] , 0, 0 )
     *
     */
    public function getCount(string $table, string $query = "", array $conditions = [], int $offset = null, int $limit = null): int
    {
        $whereClause = $query ? " WHERE $query" : "";
        $limitClause = $limit !== null ? " LIMIT :limit" : "";
        $offsetClause = $offset !== null ? " OFFSET :offset" : "";

        $countQuery = "SELECT COUNT(*) as count FROM $table$whereClause$limitClause$offsetClause";
        $stmt = $this->db->prepare($countQuery);

        foreach ($conditions as $key => $value) {
            $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        }

        if ($limit !== null) {
            $stmt->bindValue(':limit', $limit, SQLITE3_INTEGER);
        }

        if ($offset !== null) {
            $stmt->bindValue(':offset', $offset, SQLITE3_INTEGER);
        }

        $result = $stmt->execute();
        $data = $result->fetchArray(SQLITE3_ASSOC);

        return $data['count'];
    }

    /**
     * 任意の検索クエリを実行する
     *
     * @param string $query 任意のSQLクエリ
     * @param array $conditions $queryに指定された引数の内容。[arg=>value]の配列。
     * @param int|null $offset  offset
     * @param int|null $limit   limit
     *
     * @return array
     *
     */
    public function rawQuery(string $query, array $conditions = [], int $offset = null, int $limit = null): array
    {
        $limitClause = $limit !== null ? " LIMIT :limit" : "";
        $offsetClause = $offset !== null ? " OFFSET :offset" : "";

        $rawQuery = "$query$limitClause$offsetClause";
        $stmt = $this->db->prepare($rawQuery);

        foreach ($conditions as $key => $value) {
            $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        }

        if ($limit !== null) {
            $stmt->bindValue(':limit', $limit, SQLITE3_INTEGER);
        }

        if ($offset !== null) {
            $stmt->bindValue(':offset', $offset, SQLITE3_INTEGER);
        }
        $result = $stmt->execute();
        $data = [];
        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
            $data[] = $row;
        }
        return $data;
    }

    /**
     * 任意のクエリを実行する。結果は取得しない。
     *
     * @param string $query
     * @param array $conditions
     *
     * @return [type]
     *
     */
    public function execute(string $query, array $conditions = [])
    {
        $rawQuery = "$query";
        $stmt = $this->db->prepare($rawQuery);

        foreach ($conditions as $key => $value) {
            $stmt->bindValue(":$key", $value, is_int($value) ? SQLITE3_INTEGER : SQLITE3_TEXT);
        }
        $stmt->execute();
    }
}
